//****************************************************************************
//*                                                                          *
//*             modul zawiera funkcje obslugujace porty szeregowe            *
//*               (inicjowanie, posylanie i odbieranie znakow)               *
//*                                                                          *
//****************************************************************************

#include <dos.h>

//************************************************

const unsigned int  BazaZmiennychBiosa = 0x0400;
const unsigned long TimeoutNadawania   = 200000;
const unsigned long TimeoutOdbierania  = 200000;
unsigned char       BylTimeout;

//************************************************

void InicjujPortSzeregowy(unsigned char NrPortu)

//  Funkcja inicjuje port szeregowy o numerze NrPortu, parametrami ustawionymi
//  w bajcie ParametryTransmisji:
//* wartosc bajtu ParametryTransmisji = 0xFB powoduje ustawienie nastepujacych
//  parametrow transmisji:
//    - predkosc transmisji 9600 bodow,
//    - parzystosc parzysta (even),
//    - 1 bit stopu,
//    - dlugosc slowa 8 bitow.

{
  const unsigned char ParametryTransmisji = 0xFB;

  NrPortu--;
  asm {
	mov DH, 0x00
	mov DL, NrPortu
	mov AH, 0x00
	mov AL, ParametryTransmisji
	int 0x14
      }
}

//************************************************

void PoslijNaPortSzeregowy(unsigned char NrPortu, unsigned char BajtZnaku)

//  Funkcja posyla pojedynczy znak BajtZnaku przez port szeregowy o numerze
//  NrPortu:
//* przed poslaniem znaku:
//  - sprawdzane jest, czy bufor nadawania portu jest juz pusty,
//  - ustawiana jest linia RTS w stan aktywny i nastepuje sprawdzanie, czy
//    na linii CTS zostal ustawiony stan aktywny,
//* jesli ktorys z powyzszych warunkow nie jest spelniony, to sprawdzanie jest
//  ponawiane, az warunki beda spelnione lub zostanie przekroczony timeout,
//* gdy zostanie przekroczony timeout, ustawiana jest zmienna BylTimeout na
//  wartosc 1 (w przeciwnym wypadku ustawiana jest na 0),
//* w przypadku, gdy nie bylo timeouta nastepuje wyslanie znaku BajtZnaku
//  (przez wstawienie go do bufora nadawania portu)

{
  unsigned int  BazaPortu;
  unsigned int  RejestrStanuTransmisji;
  unsigned int  RejestrSterowaniaModemem;
  unsigned int  RejestrStanuModemu;
  unsigned long Licznik;

  BazaPortu                = peek(0,BazaZmiennychBiosa+(NrPortu-1)*2);
  RejestrStanuTransmisji   = BazaPortu+5;
  RejestrSterowaniaModemem = BazaPortu+4;
  RejestrStanuModemu       = BazaPortu+6;
  Licznik                  = inportb(BazaPortu);
  outportb(RejestrSterowaniaModemem,inportb(RejestrSterowaniaModemem)||2);
  Licznik                  = 0;

  if (((inportb(RejestrStanuTransmisji) & 0x20) == 0x00) ||
    ((inportb(RejestrStanuModemu) & 0x10) != 0x10))
    {
      do
	Licznik++;
      while (!((((inportb(RejestrStanuTransmisji) & 0x20) == 0x20) &&
	((inportb(RejestrStanuModemu) & 0x10) == 0x10)) ||
	(Licznik > TimeoutNadawania)));
      if (Licznik > TimeoutNadawania)
	BylTimeout = 1;
      else
	BylTimeout = 0;
    }
  else
    BylTimeout = 0;
  if (!BylTimeout)
    outportb(BazaPortu, BajtZnaku);
}

//************************************************

unsigned char OdbierzZPortuSzeregowego(unsigned char NrPortu)

//  Funkcja zwraca pojedynczy znak odebrany z portu szeregowego o numerze
//  NrPortu:
//* przed odebraniem znaku sprawdzane jest, czy w buforze odbierania portu
//  zostal skompletowany (czeka na odebranie) znak,
//* jesli powyzszy warunek nie jest spelniony, to sprawdzanie jest ponawiane,
//  az warunek bedzie spelniony lub zostanie przekroczony timeout,
//* gdy zostanie przekroczony timeout, ustawiana jest zmienna BylTimeout na
//  wartosc 1 (w przeciwnym wypadku ustawiana jest na 0),
//* w przypadku, gdy nie bylo timeouta nastepuje odczytanie znaku z bufora,
//  w przeciwnym wypadku funkcja zwraca przypadkowy bajt reprezentowany
//  przez nie zainicjowana zmienna WartoscPrzypadkowa.

{
  unsigned int  BazaPortu;
  unsigned int  RejestrStanuTransmisji;
  unsigned long Licznik;
  unsigned char Bajt;
  unsigned char WartoscPrzypadkowa;

  BazaPortu              = peek(0,BazaZmiennychBiosa+(NrPortu-1)*2);
  RejestrStanuTransmisji = BazaPortu+5;
  Licznik                = 0;

  if ((inportb(RejestrStanuTransmisji) & 0x01) == 0x00)
    {
      do
	Licznik++;
      while(!( ((inportb(RejestrStanuTransmisji) & 0x01) == 0x01) ||
	(Licznik > TimeoutOdbierania)));
      if (Licznik > TimeoutOdbierania)
	BylTimeout = 1;
      else
	BylTimeout = 0;
    }
  else
    BylTimeout = 0;
  if (!BylTimeout)
    {
      Bajt = inportb(BazaPortu);
      return(Bajt);
    }
  else
    return(WartoscPrzypadkowa);
}

//************************************************

void CzyscPortSzeregowy(unsigned char NrPortu)

//  Procedura czysci bufor odbierania portu szeregowego o numerze NrPortu:
//* sprawdzane jest, czy w buforze odbierania portu jest skompletowany znak,
//* jesli warunek jest spelniony, to nastepuje wyczyszczenie bufora odbierania
//  portu (przez odczytanie jego zawartosci).

{
  unsigned int  BazaPortu;
  unsigned int  RejestrStanuTransmisji;
  unsigned char Bajt;

  BazaPortu              = peek(0,BazaZmiennychBiosa+(NrPortu-1)*2);
  RejestrStanuTransmisji = BazaPortu+5;

  if (!((inportb(RejestrStanuTransmisji) & 0x01) == 0x00))
    inportb(BazaPortu);
}
