unit FPSerial;

{****************************************************************************}
{*                                                                          *}
{*      modul zawiera procedury i funkcje obslugujace porty szeregowe       *}
{*             (inicjowanie, posylanie i odbieranie znakow)                 *}
{*                                                                          *}
{****************************************************************************}

interface

{************************************************}

const
  BazaZmiennychBiosa : word    = $0400;
  TimeoutNadawania   : longint = 200000;
  TimeoutOdbierania  : longint = 200000;

var
  BylTimeout         : boolean;

{************************************************}

procedure InicjujPortSzeregowy(NrPortu : byte);

procedure PoslijNaPortSzeregowy(NrPortu, BajtZnaku : byte);

function OdbierzZPortuSzeregowego(NrPortu : byte) : byte;

procedure CzyscPortSzeregowy(NrPortu : byte);

{************************************************}

implementation

{************************************************}

procedure InicjujPortSzeregowy(NrPortu : byte);

{  Procedura inicjuje port szeregowy o numerze NrPortu, parametrami          }
{  ustawionymi w bajcie ParametryTransmisji:                                 }
{* wartosc bajtu ParametryTransmisji = $FB powoduje ustawienie nastepujacych }
{  parametrow transmisji:                                                    }
{    - predkosc transmisji 9600 bodow,                                       }
{    - parzystosc parzysta (even),                                           }
{    - 1 bit stopu,                                                          }
{    - dlugosc slowa 8 bitow.                                                }

const
  ParametryTransmisji : byte = $FB;

begin
  Dec(NrPortu);
  asm
    mov dh, $00
    mov dl, NrPortu
    mov ah, $00
    mov al, ParametryTransmisji
    int 14h
  end;
end;

{************************************************}

procedure PoslijNaPortSzeregowy(NrPortu, BajtZnaku : byte);

{  Procedura posyla pojedynczy znak BajtZnaku przez port szeregowy o numerze }
{  NrPortu:                                                                  }
{* przed poslaniem znaku:                                                    }
{  - sprawdzane jest, czy bufor nadawania portu jest juz pusty,              }
{  - ustawiana jest linia RTS w stan aktywny i nastepuje sprawdzanie, czy    }
{    na linii CTS zostal ustawiony stan aktywny,                             }
{* jesli ktorys z powyzszych warunkow nie jest spelniony, to sprawdzanie jest}
{  ponawiane, az warunki beda spelnione lub zostanie przekroczony timeout,   }
{* gdy zostanie przekroczony timeout, ustawiana jest zmienna BylTimeout na   }
{  wartosc logiczna true (w przeciwnym wypadku ustawiana jest na false),     }
{* w przypadku, gdy nie bylo timeouta nastepuje wyslanie znaku BajtZnaku     }
{  (przez wstawienie go do bufora nadawania portu)                           }

var
  BazaPortu                : word;
  RejestrStanuTransmisji   : word;
  RejestrSterowaniaModemem : word;
  RejestrStanuModemu       : word;
  Licznik                  : longint;

begin
  BazaPortu                      := MemW[0:BazaZmiennychBiosa+(NrPortu-1)*2];
  RejestrStanuTransmisji         := BazaPortu+5;
  RejestrSterowaniaModemem       := BazaPortu+4;
  RejestrStanuModemu             := BazaPortu+6;
  Licznik                        := Port[BazaPortu];
  Port[RejestrSterowaniaModemem] := Port[RejestrSterowaniaModemem] or 2;
  Licznik                        := 0;

  if ((Port[RejestrStanuTransmisji] and $20 = $00) or
    (Port[RejestrStanuModemu] and $10 <> $10)) then
    begin
      repeat
	Inc(Licznik);
      until (((Port[RejestrStanuTransmisji] and $20 = $20) and
	(Port[RejestrStanuModemu] and $10 = $10)) or
	(Licznik > TimeoutNadawania));
      if (Licznik > TimeoutNadawania) then
	BylTimeout := true
      else
	BylTimeout := false;
    end
  else
    BylTimeout := false;
  if (not BylTimeout) then
    Port[BazaPortu] := BajtZnaku;
end;

{************************************************}

function OdbierzZPortuSzeregowego(NrPortu : byte) : byte;

{  Funkcja zwraca pojedynczy znak odebrany z portu szeregowego o numerze     }
{  NrPortu:                                                                  }
{* przed odebraniem znaku sprawdzane jest, czy w buforze odbierania portu    }
{  zostal skompletowany (czeka na odebranie) znak,                           }
{* jesli powyzszy warunek nie jest spelniony, to sprawdzanie jest ponawiane, }
{  az warunek bedzie spelniony lub zostanie przekroczony timeout,            }
{* gdy zostanie przekroczony timeout, ustawiana jest zmienna BylTimeout na   }
{  wartosc logiczna true (w przeciwnym wypadku ustawiana jest na false),     }
{* w przypadku, gdy nie bylo timeouta nastepuje odczytanie znaku z bufora.   }

var
  BazaPortu              : word;
  RejestrStanuTransmisji : word;
  Licznik                : longint;

begin
  BazaPortu              := MemW[0:BazaZmiennychBiosa+(NrPortu-1)*2];
  RejestrStanuTransmisji := BazaPortu+5;
  Licznik                := 0;

  if (Port[RejestrStanuTransmisji] and $01 = $00) then
    begin
      repeat
	Inc(Licznik);
      until ((Port[RejestrStanuTransmisji] and $01 = $01) or
	(Licznik > TimeoutOdbierania));
      if (Licznik > TimeoutOdbierania) then
	BylTimeout := true
      else
	BylTimeout := false;
    end
  else
    BylTimeout := false;
  if (not BylTimeout) then
    OdbierzZPortuSzeregowego := Port[BazaPortu];
end;

{************************************************}

procedure CzyscPortSzeregowy(NrPortu : byte);

{  Procedura czysci bufor odbierania portu szeregowego o numerze NrPortu:    }
{* sprawdzane jest, czy w buforze odbierania portu jest skompletowany znak,  }
{* jesli warunek jest spelniony, to nastepuje wyczyszczenie bufora odbierania}
{  portu (przez odczytanie jego zawartosci).                                 }

var
  BazaPortu              : word;
  RejestrStanuTransmisji : word;
  Bajt                   : byte;

begin
  BazaPortu              := MemW[0:BazaZmiennychBiosa+(NrPortu-1)*2];
  RejestrStanuTransmisji := BazaPortu+5;

  if (Port[RejestrStanuTransmisji] and $01 <> $00) then
    Bajt := Port[BazaPortu];
end;

{************************************************}

end.